@@ -5,7 +5,7 @@ class Akismet {
 	const API_PORT = 80;
 	const MAX_DELAY_BEFORE_MODERATION_EMAIL = 86400; // One day in seconds
 
-	public static $LIMIT_NOTICES = array(
+	public static $limit_notices = array(
 		10501 => 'FIRST_MONTH_OVER_LIMIT',
 		10502 => 'SECOND_MONTH_OVER_LIMIT',
 		10504 => 'THIRD_MONTH_APPROACHING_LIMIT',
@@ -35,16 +35,16 @@ class Akismet {
 		add_filter( 'preprocess_comment', array( 'Akismet', 'auto_check_comment' ), 1 );
 		add_filter( 'rest_pre_insert_comment', array( 'Akismet', 'rest_auto_check_comment' ), 1 );
 
+		add_action( 'comment_form', array( 'Akismet', 'load_form_js' ) );
+		add_action( 'do_shortcode_tag', array( 'Akismet', 'load_form_js_via_filter' ), 10, 4 );
+
 		add_action( 'akismet_scheduled_delete', array( 'Akismet', 'delete_old_comments' ) );
 		add_action( 'akismet_scheduled_delete', array( 'Akismet', 'delete_old_comments_meta' ) );
 		add_action( 'akismet_scheduled_delete', array( 'Akismet', 'delete_orphaned_commentmeta' ) );
 		add_action( 'akismet_schedule_cron_recheck', array( 'Akismet', 'cron_recheck' ) );
 
 		add_action( 'comment_form',  array( 'Akismet',  'add_comment_nonce' ), 1 );
-
-		add_action( 'admin_head-edit-comments.php', array( 'Akismet', 'load_form_js' ) );
-		add_action( 'comment_form', array( 'Akismet', 'load_form_js' ) );
-		add_action( 'comment_form', array( 'Akismet', 'inject_ak_js' ) );
+		add_action( 'comment_form', array( 'Akismet', 'output_custom_form_fields' ) );
 		add_filter( 'script_loader_tag', array( 'Akismet', 'set_form_js_async' ), 10, 3 );
 
 		add_filter( 'comment_moderation_recipients', array( 'Akismet', 'disable_moderation_emails_if_unreachable' ), 1000, 2 );
@@ -54,9 +54,41 @@ class Akismet {
 
 		// Run this early in the pingback call, before doing a remote fetch of the source uri
 		add_action( 'xmlrpc_call', array( 'Akismet', 'pre_check_pingback' ) );
-		
+
 		// Jetpack compatibility
 		add_filter( 'jetpack_options_whitelist', array( 'Akismet', 'add_to_jetpack_options_whitelist' ) );
+		add_filter( 'jetpack_contact_form_html', array( 'Akismet', 'inject_custom_form_fields' ) );
+		add_filter( 'jetpack_contact_form_akismet_values', array( 'Akismet', 'prepare_custom_form_values' ) );
+
+		// Gravity Forms
+		add_filter( 'gform_get_form_filter', array( 'Akismet', 'inject_custom_form_fields' ) );
+		add_filter( 'gform_akismet_fields', array( 'Akismet', 'prepare_custom_form_values' ) );
+
+		// Contact Form 7
+		add_filter( 'wpcf7_form_elements', array( 'Akismet', 'append_custom_form_fields' ) );
+		add_filter( 'wpcf7_akismet_parameters', array( 'Akismet', 'prepare_custom_form_values' ) );
+
+		// Formidable Forms
+		add_filter( 'frm_filter_final_form', array( 'Akismet', 'inject_custom_form_fields' ) );
+		add_filter( 'frm_akismet_values', array( 'Akismet', 'prepare_custom_form_values' ) );
+
+		// Fluent Forms
+		/*
+		 * The Fluent Forms  hook names were updated in version 5.0.0. The last version that supported
+		 * the original hook names was 4.3.25, and version 4.3.25 was tested up to WordPress version 6.1.
+		 *
+		 * The legacy hooks are fired before the new hooks. See
+		 * https://github.com/fluentform/fluentform/commit/cc45341afcae400f217470a7bbfb15efdd80454f
+		 *
+		 * The legacy Fluent Forms hooks will be removed when Akismet no longer supports WordPress version 6.1.
+		 * This will provide compatibility with previous versions of Fluent Forms for a reasonable amount of time.
+		 */
+		add_filter( 'fluentform_form_element_start', array( 'Akismet', 'output_custom_form_fields' ) );
+		add_filter( 'fluentform_akismet_fields', array( 'Akismet', 'prepare_custom_form_values' ), 10, 2 );
+		// Current Fluent Form hooks.
+		add_filter( 'fluentform/form_element_start', array( 'Akismet', 'output_custom_form_fields' ) );
+		add_filter( 'fluentform/akismet_fields', array( 'Akismet', 'prepare_custom_form_values' ), 10, 2 );
+
 		add_action( 'update_option_wordpress_api_key', array( 'Akismet', 'updated_option' ), 10, 2 );
 		add_action( 'add_option_wordpress_api_key', array( 'Akismet', 'added_option' ), 10, 2 );
 
@@ -67,8 +99,36 @@ class Akismet {
 		return apply_filters( 'akismet_get_api_key', defined('WPCOM_API_KEY') ? constant('WPCOM_API_KEY') : get_option('wordpress_api_key') );
 	}
 
+	/**
+	 * Exchange the API key for a token that can only be used to access stats pages.
+	 *
+	 * @return string
+	 */
+	public static function get_access_token() {
+		static $access_token = null;
+
+		if ( is_null( $access_token ) ) {
+			$request_args = array( 'api_key' => self::get_api_key() );
+
+			$request_args = apply_filters( 'akismet_request_args', $request_args, 'token' );
+
+			$response = self::http_post( self::build_query( $request_args ), 'token' );
+
+			$access_token = $response[1];
+		}
+
+		return $access_token;
+	}
+
 	public static function check_key_status( $key, $ip = null ) {
-		return self::http_post( Akismet::build_query( array( 'key' => $key, 'blog' => get_option( 'home' ) ) ), 'verify-key', $ip );
+		$request_args = array(
+			'key' => $key,
+			'blog' => get_option( 'home' ),
+		);
+
+		$request_args = apply_filters( 'akismet_request_args', $request_args, 'verify-key' );
+
+		return self::http_post( self::build_query( $request_args ), 'verify-key', $ip );
 	}
 
 	public static function verify_key( $key, $ip = null ) {
@@ -86,7 +146,14 @@ class Akismet {
 	}
 
 	public static function deactivate_key( $key ) {
-		$response = self::http_post( Akismet::build_query( array( 'key' => $key, 'blog' => get_option( 'home' ) ) ), 'deactivate' );
+		$request_args = array(
+			'key' => $key,
+			'blog' => get_option( 'home' ),
+		);
+
+		$request_args = apply_filters( 'akismet_request_args', $request_args, 'deactivate' );
+
+		$response = self::http_post( self::build_query( $request_args ), 'deactivate' );
 
 		if ( $response[1] != 'deactivated' )
 			return 'failed';
@@ -213,9 +280,38 @@ class Akismet {
 		if ( ! is_null( $post ) ) {
 			// $post can technically be null, although in the past, it's always been an indicator of another plugin interfering.
 			$comment[ 'comment_post_modified_gmt' ] = $post->post_modified_gmt;
+
+			// Tags and categories are important context in which to consider the comment.
+			$comment['comment_context'] = array();
+
+			$tag_names = wp_get_post_tags( $post->ID, array( 'fields' => 'names' ) );
+
+			if ( $tag_names && ! is_wp_error( $tag_names ) ) {
+				foreach ( $tag_names as $tag_name ) {
+					$comment['comment_context'][] = $tag_name;
+				}
+			}
+
+			$category_names = wp_get_post_categories( $post->ID, array( 'fields' => 'names' ) );
+
+			if ( $category_names && ! is_wp_error( $category_names ) ) {
+				foreach ( $category_names as $category_name ) {
+					$comment['comment_context'][] = $category_name;
+				}
+			}
 		}
 
-		$response = self::http_post( Akismet::build_query( $comment ), 'comment-check' );
+		/**
+		 * Filter the data that is used to generate the request body for the API call.
+		 *
+		 * @since 5.3.1
+		 *
+		 * @param array $comment An array of request data.
+		 * @param string $endpoint The API endpoint being requested.
+		 */
+		$comment = apply_filters( 'akismet_request_args', $comment, 'comment-check' );
+
+		$response = self::http_post( self::build_query( $comment ), 'comment-check' );
 
 		do_action( 'akismet_comment_check_response', $response );
 
@@ -233,6 +329,10 @@ class Akismet {
 		if ( isset( $response[0]['x-akismet-pro-tip'] ) )
 	        $commentdata['akismet_pro_tip'] = $response[0]['x-akismet-pro-tip'];
 
+		if ( isset( $response[0]['x-akismet-guid'] ) ) {
+			$commentdata['akismet_guid'] = $response[0]['x-akismet-guid'];
+		}
+
 		if ( isset( $response[0]['x-akismet-error'] ) ) {
 			// An error occurred that we anticipated (like a suspended key) and want the user to act on.
 			// Send to moderation.
@@ -374,6 +474,10 @@ class Akismet {
 				if ( isset( self::$last_comment['akismet_pro_tip'] ) ) {
 					update_comment_meta( $comment->comment_ID, 'akismet_pro_tip', self::$last_comment['akismet_pro_tip'] );
 				}
+
+				if ( isset( self::$last_comment['akismet_guid'] ) ) {
+					update_comment_meta( $comment->comment_ID, 'akismet_guid', self::$last_comment['akismet_guid'] );
+				}
 			}
 		}
 	}
@@ -403,23 +507,28 @@ class Akismet {
 
 			$wpdb->queries = array();
 
+			$comments = array();
+
 			foreach ( $comment_ids as $comment_id ) {
-				do_action( 'delete_comment', $comment_id );
+				$comments[ $comment_id ] = get_comment( $comment_id );
+
+				do_action( 'delete_comment', $comment_id, $comments[ $comment_id ] );
 				do_action( 'akismet_batch_delete_count', __FUNCTION__ );
 			}
 
 			// Prepared as strings since comment_id is an unsigned BIGINT, and using %d will constrain the value to the maximum signed BIGINT.
-			$format_string = implode( ", ", array_fill( 0, count( $comment_ids ), '%s' ) );
+			$format_string = implode( ', ', array_fill( 0, is_countable( $comment_ids ) ? count( $comment_ids ) : 0, '%s' ) );
 
 			$wpdb->query( $wpdb->prepare( "DELETE FROM {$wpdb->comments} WHERE comment_id IN ( " . $format_string . " )", $comment_ids ) );
 			$wpdb->query( $wpdb->prepare( "DELETE FROM {$wpdb->commentmeta} WHERE comment_id IN ( " . $format_string . " )", $comment_ids ) );
 
-			clean_comment_cache( $comment_ids );
-			do_action( 'akismet_delete_comment_batch', count( $comment_ids ) );
-			
 			foreach ( $comment_ids as $comment_id ) {
-				do_action( 'deleted_comment', $comment_id );
+				do_action( 'deleted_comment', $comment_id, $comments[ $comment_id ] );
+				unset( $comments[ $comment_id ] );
 			}
+
+			clean_comment_cache( $comment_ids );
+			do_action( 'akismet_delete_comment_batch', is_countable( $comment_ids ) ? count( $comment_ids ) : 0 );
 		}
 
 		if ( apply_filters( 'akismet_optimize_table', ( mt_rand(1, 5000) == 11), $wpdb->comments ) ) // lucky number
@@ -449,7 +558,7 @@ class Akismet {
 				do_action( 'akismet_batch_delete_count', __FUNCTION__ );
 			}
 
-			do_action( 'akismet_delete_commentmeta_batch', count( $comment_ids ) );
+			do_action( 'akismet_delete_commentmeta_batch', is_countable( $comment_ids ) ? count( $comment_ids ) : 0 );
 		}
 
 		if ( apply_filters( 'akismet_optimize_table', ( mt_rand(1, 5000) == 11), $wpdb->commentmeta ) ) // lucky number
@@ -497,16 +606,50 @@ class Akismet {
 	public static function get_user_comments_approved( $user_id, $comment_author_email, $comment_author, $comment_author_url ) {
 		global $wpdb;
 
-		if ( !empty( $user_id ) )
-			return (int) $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(*) FROM {$wpdb->comments} WHERE user_id = %d AND comment_approved = 1", $user_id ) );
+		/**
+		 * Which comment types should be ignored when counting a user's approved comments?
+		 *
+		 * Some plugins add entries to the comments table that are not actual
+		 * comments that could have been checked by Akismet. Allow these comments
+		 * to be excluded from the "approved comment count" query in order to
+		 * avoid artificially inflating the approved comment count.
+		 *
+		 * @param array $comment_types An array of comment types that won't be considered
+		 *                             when counting a user's approved comments.
+		 *
+		 * @since 4.2.2
+		 */
+		$excluded_comment_types = apply_filters( 'akismet_excluded_comment_types', array() );
 
-		if ( !empty( $comment_author_email ) )
-			return (int) $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(*) FROM {$wpdb->comments} WHERE comment_author_email = %s AND comment_author = %s AND comment_author_url = %s AND comment_approved = 1", $comment_author_email, $comment_author, $comment_author_url ) );
+		$comment_type_where = '';
+
+		if ( is_array( $excluded_comment_types ) && ! empty( $excluded_comment_types ) ) {
+			$excluded_comment_types = array_unique( $excluded_comment_types );
+
+			foreach ( $excluded_comment_types as $excluded_comment_type ) {
+				$comment_type_where .= $wpdb->prepare( ' AND comment_type <> %s ', $excluded_comment_type );
+			}
+		}
+
+		if ( ! empty( $user_id ) ) {
+			return (int) $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(*) FROM {$wpdb->comments} WHERE user_id = %d AND comment_approved = 1" . $comment_type_where, $user_id ) );
+		}
+
+		if ( ! empty( $comment_author_email ) ) {
+			return (int) $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(*) FROM {$wpdb->comments} WHERE comment_author_email = %s AND comment_author = %s AND comment_author_url = %s AND comment_approved = 1" . $comment_type_where, $comment_author_email, $comment_author, $comment_author_url ) );
+		}
 
 		return 0;
 	}
 
-	// get the full comment history for a given comment, as an array in reverse chronological order
+	/**
+	 * Get the full comment history for a given comment, as an array in reverse chronological order.
+	 * Each entry will have an 'event', a 'time', and possible a 'message' member (if the entry is old enough).
+	 * Some entries will also have a 'user' or 'meta' member.
+	 *
+	 * @param int $comment_id The relevant comment ID.
+	 * @return array|bool An array of history events, or false if there is no history.
+	 */
 	public static function get_comment_history( $comment_id ) {
 		$history = get_comment_meta( $comment_id, 'akismet_history', false );
 		if ( empty( $history ) || empty( $history[ 0 ] ) ) {
@@ -545,6 +688,10 @@ class Akismet {
 		$history[] = array( 'time' => 445856425, 'event' => 'status-spam', 'user' => 'sam' );
 		$history[] = array( 'time' => 445856426, 'event' => 'status-hold', 'user' => 'sam' );
 		$history[] = array( 'time' => 445856427, 'event' => 'status-approve', 'user' => 'sam' );
+		$history[] = array( 'time' => 445856427, 'event' => 'webhook-spam' );
+		$history[] = array( 'time' => 445856427, 'event' => 'webhook-ham' );
+		$history[] = array( 'time' => 445856427, 'event' => 'webhook-spam-noaction' );
+		$history[] = array( 'time' => 445856427, 'event' => 'webhook-ham-noaction' );
 		*/
 		
 		usort( $history, array( 'Akismet', '_cmp_time' ) );
@@ -611,7 +758,9 @@ class Akismet {
 		if ( self::is_test_mode() )
 			$c['is_test'] = 'true';
 
-		$response = self::http_post( Akismet::build_query( $c ), 'comment-check' );
+		$c = apply_filters( 'akismet_request_args', $c, 'comment-check' );
+
+		$response = self::http_post( self::build_query( $c ), 'comment-check' );
 
 		if ( ! empty( $response[1] ) ) {
 			return $response[1];
@@ -625,8 +774,6 @@ class Akismet {
 		
 		$api_response = self::check_db_comment( $id, $recheck_reason );
 
-		delete_comment_meta( $id, 'akismet_rechecking' );
-
 		if ( is_wp_error( $api_response ) ) {
 			// Invalid comment ID.
 		}
@@ -654,6 +801,8 @@ class Akismet {
 			);
 		}
 
+		delete_comment_meta( $id, 'akismet_rechecking' );
+
 		return $api_response;
 	}
 
@@ -681,6 +830,17 @@ class Akismet {
 		if ( get_comment_meta( $comment->comment_ID, 'akismet_rechecking' ) )
 			return;
 		
+		if ( function_exists( 'getallheaders' ) ) {
+			$request_headers = getallheaders();
+
+			foreach ( $request_headers as $header => $value ) {
+				if ( strtolower( $header ) == 'x-akismet-webhook' ) {
+					// This change is due to a webhook request.
+					return;
+				}
+			}
+		}
+
 		// Assumption alert:
 		// We want to submit comments to Akismet only when a moderator explicitly spams or approves it - not if the status
 		// is changed automatically by another plugin.  Unfortunately WordPress doesn't provide an unambiguous way to
@@ -773,7 +933,9 @@ class Akismet {
 			$comment->comment_post_modified_gmt = $post->post_modified_gmt;
 		}
 
-		$response = Akismet::http_post( Akismet::build_query( $comment ), 'submit-spam' );
+		$comment = apply_filters( 'akismet_request_args', $comment, 'submit-spam' );
+
+		$response = self::http_post( self::build_query( $comment ), 'submit-spam' );
 
 		update_comment_meta( $comment_id, 'akismet_user_result', 'true' );
 
@@ -831,7 +993,9 @@ class Akismet {
 			$comment->comment_post_modified_gmt = $post->post_modified_gmt;
 		}
 
-		$response = self::http_post( Akismet::build_query( $comment ), 'submit-ham' );
+		$comment = apply_filters( 'akismet_request_args', $comment, 'submit-ham' );
+
+		$response = self::http_post( self::build_query( $comment ), 'submit-ham' );
 
 		update_comment_meta( $comment_id, 'akismet_user_result', 'false' );
 
@@ -982,7 +1146,7 @@ class Akismet {
 		if ( is_user_logged_in() )
 			return false;
 	
-		return ( get_option( 'akismet_strictness' ) === '1'  );
+		return ( get_option( 'akismet_strictness' ) === '1' );
 	}
 
 	public static function get_ip_address() {
@@ -1064,6 +1228,7 @@ class Akismet {
 
 	// return a comma-separated list of role names for the given user
 	public static function get_user_roles( $user_id ) {
+		$comment_user = null;
 		$roles = false;
 
 		if ( !class_exists('WP_User') )
@@ -1072,7 +1237,7 @@ class Akismet {
 		if ( $user_id > 0 ) {
 			$comment_user = new WP_User( $user_id );
 			if ( isset( $comment_user->roles ) )
-				$roles = join( ',', $comment_user->roles );
+				$roles = implode( ',', $comment_user->roles );
 		}
 
 		if ( is_multisite() && is_super_admin( $user_id ) ) {
@@ -1080,7 +1245,7 @@ class Akismet {
 				$roles = 'super_admin';
 			} else {
 				$comment_user->roles[] = 'super_admin';
-				$roles = join( ',', $comment_user->roles );
+				$roles = implode( ',', $comment_user->roles );
 			}
 		}
 
@@ -1132,10 +1297,12 @@ class Akismet {
 		if ( ! empty( self::$prevent_moderation_email_for_these_comments ) && ! empty( $emails ) ) {
 			$comment = get_comment( $comment_id );
 
-			foreach ( self::$prevent_moderation_email_for_these_comments as $possible_match ) {
-				if ( self::comments_match( $possible_match, $comment ) ) {
-					update_comment_meta( $comment_id, 'akismet_delayed_moderation_email', true );
-					return array();
+			if ( $comment ) {
+				foreach ( self::$prevent_moderation_email_for_these_comments as $possible_match ) {
+					if ( self::comments_match( $possible_match, $comment ) ) {
+						update_comment_meta( $comment_id, 'akismet_delayed_moderation_email', true );
+						return array();
+					}
 				}
 			}
 		}
@@ -1165,13 +1332,12 @@ class Akismet {
 		$akismet_ua = sprintf( 'WordPress/%s | Akismet/%s', $GLOBALS['wp_version'], constant( 'AKISMET_VERSION' ) );
 		$akismet_ua = apply_filters( 'akismet_ua', $akismet_ua );
 
-		$content_length = strlen( $request );
-
-		$api_key   = self::get_api_key();
 		$host      = self::API_HOST;
+		$api_key   = self::get_api_key();
 
-		if ( !empty( $api_key ) )
-			$host = $api_key.'.'.$host;
+		if ( $api_key ) {
+			$request = add_query_arg( 'api_key', $api_key, $request );
+		}
 
 		$http_host = $host;
 		// use a specific IP if provided
@@ -1275,55 +1441,123 @@ class Akismet {
 			'usage-limit',
 			'upgrade-plan',
 			'upgrade-url',
+			'upgrade-type',
 		);
 
-		foreach( $alert_header_names as $alert_header_name ) {
+		foreach ( $alert_header_names as $alert_header_name ) {
 			$value = null;
-			if ( isset( $response[0][$alert_header_prefix . $alert_header_name] ) ) {
-				$value = $response[0][$alert_header_prefix . $alert_header_name];
+			if ( isset( $response[0][ $alert_header_prefix . $alert_header_name ] ) ) {
+				$value = $response[0][ $alert_header_prefix . $alert_header_name ];
 			}
 
 			$option_name = $alert_option_prefix . str_replace( '-', '_', $alert_header_name );
 			if ( $value != get_option( $option_name ) ) {
 				if ( ! $value ) {
 					delete_option( $option_name );
-				}
-				else {
+				} else {
 					update_option( $option_name, $value );
 				}
 			}
 		}
 	}
 
-	public static function load_form_js() {
-		if ( function_exists( 'is_amp_endpoint' ) && is_amp_endpoint() ) {
-			return;
-		}
-
-		if ( ! self::get_api_key() ) {
-			return;
-		}
-
-		wp_register_script( 'akismet-form', plugin_dir_url( __FILE__ ) . '_inc/form.js', array(), AKISMET_VERSION, true );
-		wp_enqueue_script( 'akismet-form' );
-	}
-	
 	/**
-	 * Mark form.js as deferred. Because nothing depends on it, it can run at any time
+	 * Mark akismet-frontend.js as deferred. Because nothing depends on it, it can run at any time
 	 * after it's loaded, and the browser won't have to wait for it to load to continue
 	 * parsing the rest of the page.
 	 */
 	public static function set_form_js_async( $tag, $handle, $src ) {
-		if ( 'akismet-form' !== $handle ) {
+		if ( 'akismet-frontend' !== $handle ) {
 			return $tag;
 		}
-		
+
 		return preg_replace( '/^<script /i', '<script defer ', $tag );
 	}
-	
-	public static function inject_ak_js( $post_id ) {
-		echo '<input type="hidden" id="ak_js" name="ak_js" value="' . mt_rand( 0, 250 ) . '"/>';
-		echo '<textarea name="ak_hp_textarea" cols="45" rows="8" maxlength="100" style="display: none !important;"></textarea>';
+
+	public static function get_akismet_form_fields() {
+		$fields = '';
+
+		$prefix = 'ak_';
+
+		// Contact Form 7 uses _wpcf7 as a prefix to know which fields to exclude from comment_content.
+		if ( 'wpcf7_form_elements' === current_filter() ) {
+			$prefix = '_wpcf7_ak_';
+		}
+
+		$fields .= '<p style="display: none !important;" class="akismet-fields-container" data-prefix="' . esc_attr( $prefix ) . '">';
+		$fields .= '<label>&#916;<textarea name="' . $prefix . 'hp_textarea" cols="45" rows="8" maxlength="100"></textarea></label>';
+
+		if ( ! function_exists( 'amp_is_request' ) || ! amp_is_request() ) {
+			// Keep track of how many ak_js fields are in this page so that we don't re-use
+			// the same ID.
+			static $field_count = 0;
+
+			$field_count++;
+
+			$fields .= '<input type="hidden" id="ak_js_' . $field_count . '" name="' . $prefix . 'js" value="' . mt_rand( 0, 250 ) . '"/>';
+			$fields .= '<script>document.getElementById( "ak_js_' . $field_count . '" ).setAttribute( "value", ( new Date() ).getTime() );</script>';
+		}
+
+		$fields .= '</p>';
+
+		return $fields;
+	}
+
+	public static function output_custom_form_fields( $post_id ) {
+		if ( 'fluentform/form_element_start' === current_filter() && did_action( 'fluentform_form_element_start' ) ) {
+			// Already did this via the legacy filter.
+			return;
+		}
+
+		// phpcs:ignore WordPress.Security.EscapeOutput
+		echo self::get_akismet_form_fields();
+	}
+
+	public static function inject_custom_form_fields( $html ) {
+		$html = str_replace( '</form>', self::get_akismet_form_fields() . '</form>', $html );
+
+		return $html;
+	}
+
+	public static function append_custom_form_fields( $html ) {
+		$html .= self::get_akismet_form_fields();
+
+		return $html;
+	}
+
+	/**
+	 * Ensure that any Akismet-added form fields are included in the comment-check call.
+	 *
+	 * @param array $form
+	 * @param array $data Some plugins will supply the POST data via the filter, since they don't
+	 *                    read it directly from $_POST.
+	 * @return array $form
+	 */
+	public static function prepare_custom_form_values( $form, $data = null ) {
+		if ( 'fluentform/akismet_fields' === current_filter() && did_filter( 'fluentform_akismet_fields' ) ) {
+			// Already updated the form fields via the legacy filter.
+			return $form;
+		}
+
+		if ( is_null( $data ) ) {
+			// phpcs:ignore WordPress.Security.NonceVerification.Missing
+			$data = $_POST;
+		}
+
+		$prefix = 'ak_';
+
+		// Contact Form 7 uses _wpcf7 as a prefix to know which fields to exclude from comment_content.
+		if ( 'wpcf7_akismet_parameters' === current_filter() ) {
+			$prefix = '_wpcf7_ak_';
+		}
+
+		foreach ( $data as $key => $val ) {
+			if ( 0 === strpos( $key, $prefix ) ) {
+				$form[ 'POST_ak_' . substr( $key, strlen( $prefix ) ) ] = $val;
+			}
+		}
+
+		return $form;
 	}
 
 	private static function bail_on_activation( $message, $deactivate = true ) {
@@ -1371,7 +1605,7 @@ p {
 	public static function view( $name, array $args = array() ) {
 		$args = apply_filters( 'akismet_view_arguments', $args, $name );
 		
-		foreach ( $args AS $key => $val ) {
+		foreach ( $args as $key => $val ) {
 			$$key = $val;
 		}
 		
@@ -1446,6 +1680,7 @@ p {
 	}
 
 	public static function pre_check_pingback( $method ) {
+		$pingback_args = array();
 		if ( $method !== 'pingback.ping' )
 			return;
 
@@ -1470,7 +1705,7 @@ p {
 
 			if ( 0 === $call_count ) {
 				// Only pass along the number of entries in the multicall the first time we see it.
-				$multicall_count = count( $wp_xmlrpc_server->message->params );
+				$multicall_count = is_countable( $wp_xmlrpc_server->message->params ) ? count( $wp_xmlrpc_server->message->params ) : 0;
 			}
 
 			/*
@@ -1636,4 +1871,65 @@ p {
 			) . '</p>'
 		);
 	}
+
+	public static function load_form_js() {
+		if (
+			! is_admin()
+			&& ( ! function_exists( 'amp_is_request' ) || ! amp_is_request() )
+			&& self::get_api_key()
+			) {
+			wp_register_script( 'akismet-frontend', plugin_dir_url( __FILE__ ) . '_inc/akismet-frontend.js', array(), filemtime( plugin_dir_path( __FILE__ ) . '_inc/akismet-frontend.js' ), true );
+			wp_enqueue_script( 'akismet-frontend' );
+		}
+	}
+
+	/**
+	 * Add the form JavaScript when we detect that a supported form shortcode is being parsed.
+	 */
+	public static function load_form_js_via_filter( $return_value, $tag, $attr, $m ) {
+		if ( in_array( $tag, array( 'contact-form', 'gravityform', 'contact-form-7', 'formidable', 'fluentform' ) ) ) {
+			self::load_form_js();
+		}
+
+		return $return_value;
+	}
+
+	/**
+	 * Was the last entry in the comment history created by Akismet?
+	 *
+	 * @param int $comment_id The ID of the comment.
+	 * @return bool
+	 */
+	public static function last_comment_status_change_came_from_akismet( $comment_id ) {
+		$history = self::get_comment_history( $comment_id );
+
+		if ( empty( $history ) ) {
+			return false;
+		}
+
+		$most_recent_history_event = $history[0];
+
+		if ( ! isset( $most_recent_history_event['event'] ) ) {
+			return false;
+		}
+
+		$akismet_history_events = array(
+			'check-error',
+			'cron-retry-ham',
+			'cron-retry-spam',
+			'check-ham',
+			'check-spam',
+			'recheck-error',
+			'recheck-ham',
+			'recheck-spam',
+			'webhook-ham',
+			'webhook-spam',
+		);
+
+		if ( in_array( $most_recent_history_event['event'], $akismet_history_events ) ) {
+			return true;
+		}
+
+		return false;
+	}
 }
